(* Umwandlung: Dezimalzahlen in rmische Zahlen und umgekehrt
   Stichwort: Algorithmus umsetzten
*)


unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TForm1 = class(TForm)
    Edit1: TEdit;
    Edit2: TEdit;
    Button1: TButton;
    Label1: TLabel;
    procedure Edit1Change(Sender: TObject);
    procedure Edit2Change(Sender: TObject);
    procedure Edit2KeyPress(Sender: TObject; var Key: Char);
    procedure Edit1KeyPress(Sender: TObject; var Key: Char);
    procedure Button1Click(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

{              Rmische Zahlen:Algorithmus
              ===========================
  Rmische Zahlen Algorithmus: Beispiel 1964
   1.Tausender verarbeiten 1964=M+964   (M+Rest)
       Rest >=900. Inversion  1964=CM+64   (MCM+Rest)
   2. Fnfhunderter verarbeiten
      Rest>=400: Inversion
   3. Hunderter verarbeiten
      Rest>=90:   Inversion
   4. Fnfziger verarbeiten  1964=MCM+L+14
      Rest>=40: Inversion
   5. Zehner verarbeiten     1964=MCM+L+X+4
      Rest>=9: Inversion
   6. Fnfer verarbeiten
      Rest>=4: Inversion     1964=MCM+L+X+IV
   7. Einer verarbeiten
   ------------- Delphi-Funktion-------------
     in Pascal: Hilfsvariable var result:string
                letzter Funktionsaufruf "teil:=result"
                bzw. DezInRoem:=resulr"}
function DezInRoem(const s:string):string;
  var n:Integer;
  function teil(const groessenordnung1,groessenordnung2:integer;
                var rest:integer;
                const zeichen1,zeichen2:char):string;
    begin
      result:='';
      while rest>=groessenordnung1 do Begin
        dec(rest,groessenordnung1); //rest:=rest-groessenordnung;
        result:=result+zeichen1;  //z.B. +'M'
      End;
      if rest>=groessenordnung2 then Begin
        dec(rest,groessenordnung2);
        result:=result+zeichen2+zeichen1; //z.B. +'CM'
      End;
    end;
begin
  n:=strToInt(s);
  result:=''; //in Delphi automatisch
  result:=result+teil(1000,900,n,'M','C');
  result:=result+teil(500,400,n,'D','C');
  result:=result+teil(100,90,n,'C','X');
  result:=result+teil(50,40,n,'L','X');
  result:=result+teil(10,9,n,'X','I');
  result:=result+teil(5,4,n,'V','I');
  result:=result+teil(1,1,n,'I','I');
end;
(* Das ganze komprimiert:
function DecInRoem(d:integer): string;
  const
    aRomans: array[1..13] of string =
      ('I','IV','V','IX','X','XL','L','XC','C','CD','D','CM','M' );
    aArabics: array[1..13] of integer=
      (1,4,5,9,10,40,50,90,100,400,500,900,1000);
  var i:integer;
begin
  for i := 13 downto 1 do
    while d>=aArabics[i] do Begin
      d:=d-aArabics[i];
      result:=result+aRomans[i];
    End;
end;*)

function RoemInDez(s:string):string;
    var n:integer;
  procedure teil(var s:string; var n:integer; const ZZ:string; const zahl:integer);
      var p:integer;
    begin
      while pos(ZZ,s)>0 do Begin
        p:=pos(ZZ,s);
        inc(n,zahl);
        Delete(s,p,length(ZZ));
      End;
    end;
  function typischerFehler(const s:string):string;
    begin
      if pos('IC',s)>0 then result:='IC=XCIX (90+9)' else
      if pos('IM',s)>0 then result:='IM=CMXCIX (900+90+9)' else
      if pos('XM',s)>0 then result:='XM=CMXC (900+90)' else
      // ... "VM" etc. Der Phantasie sind keine Grenzen gesetzt
      result:='';
    end;
begin
  if typischerFehler(s)>'' then Begin
    result:='Fehler: '+typischerFehler(s);
    exit;
  End;
  n:=0;
  teil(s,n,'CM',900);
  teil(s,n,'M',1000);
  teil(s,n,'CD',400);
  teil(s,n,'D',500);
  teil(s,n,'XC',90);
  teil(s,n,'C',100);
  teil(s,n,'XL',40);
  teil(s,n,'L',50);
  teil(s,n,'IX',9);
  teil(s,n,'X',10);
  teil(s,n,'IV',4);
  teil(s,n,'V',5);
  teil(s,n,'I',1);
  result:=intToStr(n);
  s:=trim(s);
  if s>'' then result:=result+' Rest:'+s;
end;


procedure TForm1.Edit1Change(Sender: TObject);
begin
   if edit1.Focused then edit2.text:='';
end;

procedure TForm1.Edit2Change(Sender: TObject);
begin
  if edit2.Focused then edit1.text:='';
end;

procedure TForm1.Edit2KeyPress(Sender: TObject; var Key: Char);
begin
  if key=#13 then Begin
    edit1.text:=RoemInDez(edit2.text);
    //Korrektur der Eingabe
    if pos('F',edit1.text)=0 then //Kein _F_ehler
      edit2.text:=DezInRoem(edit1.text);
    //changed => edit1.text=''. Deshalb zum 2. Mal
    edit1.text:=RoemInDez(edit2.text);
  End;
end;

procedure TForm1.Edit1KeyPress(Sender: TObject; var Key: Char);
begin
  if key=#13 then edit2.text:=DezInRoem(edit1.text);
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  close
end;

end.
